using System;
using System.Xml;
using System.Data;
using System.Data.SqlClient;
using Vha.Vss.Utility;
namespace Vha.Vss.Utility
{
 
  ///<summary>
  ///The DBConnection class represents all the properties required to make a connection to a database.
  ///It is the C# equivilent of the &lt;DBConnection&gt; tag in the .config file.
  ///</summary>
  public class DBConnection : VhaConnection
  {
    /// <summary>
    /// Private member required by the Designer.
    /// </summary>
    private void InitializeComponent ()
    {
    }

    String dbConnectionName;

    ///<summary>
    ///Default constructor.
    ///</summary>
    ///<Params>
    ///None.
    ///</Params>
    ///<returns type="void">
    ///</returns>
    ///<remarks>
    ///Used primarily by admin tools for new .config file entries.  Initializes all internal values to null.
    ///</remarks>
    ///<Exceptions>
    ///None.
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///using Microsoft.eSupport.Utility.Configuration
    ///...
    ///DBConnection myDBConn = new DBConnection();
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    /// <seealso cref="DBConnection(XmlNode)"/>
    ///</References>
    public DBConnection() {}  // end of ctor DBConnection


    ///<summary>
    ///Constructs a DBConnection from a valid XmlNode.  This constructor is called by the 
    ///Component object to construct and return the fully loaded DBConnection object to 
    ///a calling application.
    ///</summary>
    ///<Params>
    ///<param type="XmlNode" name="DBConnectionNode" Required="Mandatory">
    ///<para>This XmlNode object must be reflective of the following XML structure...
    ///</para>
    ///<XMLTree>
    /// <DBConnection 
    /// DBConnectionName="eSupportLog" 
    /// DatabaseName="dbesLog"
    /// ServerName="sixstring"
    /// UserId="sa"
    /// Password="sa" >
    /// </DBConnection>
    ///</XMLTree>
    ///</param>
    ///</Params>
    ///<returns type="void">
    ///</returns>
    ///<remarks>
    ///<para>This constructor is typically not used outside of the utility dll.
    ///</para>
    ///</remarks>
    ///<Exceptions>
    ///None.
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///public void Foo()
    ///{
    ///  XmlNodeList DBConnections = xmlDoc.GetElementsByTagName("DBConnection");
    ///  for (int i=0; i&lt;DBConnections.Count; i++)
    ///  {
    ///    if(DBConnections.Item(i).Attributes.GetNamedItem("DBConnectionName").Value == DBConnectionName)
    ///    {
    ///      return new DBConnection(DBConnections.Item(i));
    ///    }
    ///  }
    ///}
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    ///</References>
    public DBConnection(XmlNode DBConnectionNode)
    {
      XmlAttributeCollection Atts = DBConnectionNode.Attributes;

      //  This constructor is called by the Component object when reading a .config file
      if (Atts.GetNamedItem("DBConnectionName") != null) 
        dbConnectionName  = Atts.GetNamedItem("DBConnectionName").Value ;

      //  This part is going to extract the password and if it is in the 
      //  format [xxxx] it will be considered encrypted and will then be decrypted.
      if (Atts.GetNamedItem("ConnectionString") != null) 
      {
        String prompt = "Password=";
        String connStr, password;
        connStr = Atts.GetNamedItem("ConnectionString").Value;
        int i = connStr.IndexOf(prompt);
        
        if(i>=0)    // is there a Password specified
        {
          i += prompt.Length;
          int j = connStr.IndexOf(";", i);            // find the end of it.
          if (j<0) j = connStr.Length;                // deal with it being the end of the string.
          password = connStr.Substring(i,j-i);        // extract the password value
          if (password.StartsWith("[") && password.EndsWith("]"))   // is it encrypted?
          {
            DataEncoder de = new DataEncoder();       // decrypt it
            password = de.DecodeString(password.Substring(1, password.Length-2));
          }
          //  Now rebuild the connection string.
          connStr = connStr.Substring(0,i) + password + connStr.Substring(j);
        }
        Connection.ConnectionString = connStr;
      }
      // *** Not supported as writeable in B2
      //    if (Atts.GetNamedItem("ConnectionTimeout")  != null) 
      //      Connection.ConnectionTimeout  = Convert.ToInt32(Atts.GetNamedItem("ConnectionTimeout").Value);

      // *** Not supported as writeable in B2
      //    if (Atts.GetNamedItem("Database")     != null) 
      //      Connection.Database = Atts.GetNamedItem("Database").Value;
      //    if (Atts.GetNamedItem("DataSource")     != null) 
      //      Connection.DataSource = Atts.GetNamedItem("DataSource").Value;

      // *** Not supported at all in B2
      //    if (Atts.GetNamedItem("IsolationLevel")   != null) 
      //      Connection.i.IsolationLevel = (IsolationLevel)Enum.FromString(typeof(IsolationLevel), Atts.GetNamedItem("IsolationLevel").Value);
      //    if (Atts.GetNamedItem("Password")     != null) 
      //      Connection.Password = Atts.GetNamedItem("Password").Value;
      //    if (Atts.GetNamedItem("UserID")       != null) 
      //      Connection.UserID = Atts.GetNamedItem("UserID").Value;

    } // end of ctor DBConnection


    /// <summary>
    /// This function tests a DBConnection object's ability to connect to a database. It connects
    /// then runs a simple query ("select @@Version").
    /// </summary>
    ///<Params>
    /// <param type="out String" name="Output"  Required="Mandatory"> 
    /// Results of the query are returned in this string variable.
    /// </param>
    ///</Params>
    ///<returns type="bool">
    ///True indicates a successful connection and the Output parameter contains the results.  
    ///False means the connection failed and the Output parameter will contain the exception message.
    ///</returns>
    ///<remarks>
    ///Used by the Installer classes to test DBConnection in the .config files.
    ///</remarks>
    ///<Exceptions>
    ///None.
    ///</Exceptions>
    ///<Examples>
    ///<example language="C#">
    ///<code>
    ///[to be supplied]
    ///</code>
    ///</example>
    ///</Examples>
    ///<References>
    ///</References>
    public bool IsValid(out String Output)
    {
      bool rc = false;
      try
      {
        String s="";
        //  Test the DBConnection object...
        Command.CommandType = CommandType.Text;
        this.ExecuteSql(ref s, "select @@Version", CommandType.Text);
        Output = "Version of SQL Server: " + s;
        rc = true;
      }
      catch(Exception e)
      {
        Output = "[" + e.GetType().ToString() + "] - " + e.Message;
      }
      return rc;
    }

  } // end of class DBConnection
} // end of namespace Microsoft.eSupport.Utility.Configuration
