namespace Vha.Vss.Utility
{
  using System;
  using System.IO;
  using System.Security;
  using System.Security.Cryptography;

  /// <summary>
  ///   The DataEncoder class provides simple to use encoding/decoding methods
  ///   for securing data.  The pass phrase can be anything the component desires,
  ///   but must be the same for encoding and decoding.
  /// </summary>
  public class DataEncoder
  {
    byte[]  SymKey;
    byte[]  SymIV;

    ///<summary>
    /// Constructs an object instance using a default pass-phrase of 'eSupport'.
    ///</summary>
    ///<Params>None.
    ///</Params>
    public DataEncoder() : this("VeteransHealthAdministration") {}    

    ///<summary>
    /// Constructs an object instance using an explicitly supplied pass-phrase.
    ///</summary>
    ///<Params>
    ///  <param type="string" name="passPhrase" Required="">
    ///  Contains the pass phrase that will be used with the DES encryption algorithm.
    ///  </param>
    ///</Params>
    public DataEncoder(string passPhrase)
    {
      int nChar;

      // Create a Key and Initialization Vector from the passPhrase
      SymKey = new byte[8];
      SymIV = new byte[8];

      for (nChar = 0; nChar < passPhrase.Length; nChar++)
      {
        int nKeyByte = nChar % 8;
        int nIVByte = (nChar + 3) % 8;
        byte cb = (byte)passPhrase[nChar];

        SymKey[nKeyByte] ^= cb;
        SymIV[nIVByte] ^= cb;
      }
    }

    ///<summary>
    /// This method encodes a chunk of binary data with the pass phrase
    ///</summary>
    ///<Params>
    ///  <param type="byte[]" name="bp" Required="">
    ///  Data to be encoded
    ///  </param>
    ///</Params>
    ///<returns type="byte[]">
    ///  Returns the encoded Data
    ///</returns>
    public byte[] EncodeData(byte[] bp)
    {
      // Create the DES encryption service and encrypt our data out to a memory stream
      DESCryptoServiceProvider des = new DESCryptoServiceProvider();
      ICryptoTransform desencrypt = des.CreateEncryptor(SymKey,SymIV);
      MemoryStream myStream = new MemoryStream();
      CryptoStream cryptostream = new CryptoStream(myStream, desencrypt, CryptoStreamMode.Write);

      cryptostream.Write( bp, 0, bp.Length );
      cryptostream.Close();


      return myStream.ToArray();
    }

    ///<summary>
    /// This method decodes a chunk of binary data with the pass phrase
    ///</summary>
    ///<Params>
    ///  <param type="byte[]" name="bp" Required="">
    ///  Data to be decoded
    ///  </param>
    ///</Params>
    ///<returns type="byte[]">
    ///  Returns the decoded data
    ///</returns>
    public byte[] DecodeData(byte[] bp)
    {
      // Create the DES encryption service and encrypt our data out to a memory stream
      DESCryptoServiceProvider des = new DESCryptoServiceProvider();
      ICryptoTransform desdecrypt = des.CreateDecryptor(SymKey,SymIV);
      MemoryStream myStream = new MemoryStream();
      CryptoStream cryptostream = new CryptoStream(myStream, desdecrypt, CryptoStreamMode.Write);

      cryptostream.Write( bp, 0, bp.Length );
      cryptostream.Close();

      return myStream.ToArray();
    }

    ///<summary>
    /// This method encodes a string with the pass phrase
    ///</summary>
    ///<Params>
    ///  <param type="string" name="source" Required="">
    ///  String to be encoded
    ///  </param>
    ///</Params>
    ///<returns type="string">
    ///  Returns the encoded string
    ///</returns>
    public string EncodeString(string source)
    {
      byte[] bp, bpEncoded;

      // First convert the string into bytes
      bp = StringToByteArray( source );

      // Now encode the data
      bpEncoded = EncodeData( bp );

      // Now create a ASCII compatible string to return
      return this.ConvertBytesToASCIIString( bpEncoded );

    } //*** EncodeString
    
    ///<summary>
    /// This method decodes a string with the pass phrase
    ///</summary>
    ///<Params>
    ///  <param type="string" name="source" Required="">
    ///  String to be decoded
    ///  </param>
    ///</Params>
    ///<returns type="string">
    ///  Returns the decoded string
    ///</returns>
    public string DecodeString(string source)
    {
      byte[] bp, bpDecoded;

      // First convert the string (presumable our ASCII dohickey)
      bp = ConvertASCIIStringToBytes( source );

      // Now decode
      bpDecoded = DecodeData( bp );

      // Now convert back to a normal Unicode string
      return this.ByteArrayToString( bpDecoded );

    } //*** DecodeString

    ///<summary>
    /// This method converts a string into an byte array
    ///</summary>
    ///<Params>
    ///  <param type="string" name="source" Required="">
    ///  String to be converted
    ///  </param>
    ///</Params>
    ///<returns type="byte[]">
    ///  Returns the byte array
    ///</returns>
    public byte[] StringToByteArray(string source)
    {
      int i, len;
      char[] cp = source.ToCharArray();

      // change the pass phrase into a form we can use
      len = cp.Length;

      // convert to byte array -  discard upper byte since is usually 0 in Eng.
      byte[] bp = new byte[len * 2];

      for (i = 0; i<len; i++)
      {
        bp[i*2] = (byte)((cp[i] & 0xFF00) >> 8);
        bp[i*2 + 1] = (byte)(cp[i] & 0x00FF);
      }

      return bp;

    } //*** StringToByteArray

    ///<summary>
    /// This method converts a byte array into a string
    ///</summary>
    ///<Params>
    ///  <param type="byte[]" name="source" Required="">
    ///  bytes to be converted into a string
    ///  </param>
    ///</Params>
    ///<returns type="string">
    ///  Returns the string formed from the bytes
    ///</returns>
    public string ByteArrayToString(byte[] source)
    {
      System.Text.StringBuilder sb = new System.Text.StringBuilder(source.Length / 2);
      int i;

      for (i = 0; i<source.Length; i+=2)
      {
        int val = source[i];
        val <<= 8;
        val += source[i+1];

        sb.Append( Convert.ToChar(val) );
      }

      return sb.ToString();

    } //*** ByteArrayToString

    ///<summary>
    /// Converts a byte array into an ASCII compatible string
    ///</summary>
    ///<Params>
    ///  <param type="byte[]" name="bp" Required="">
    ///  Byte array to convert
    ///  </param>
    ///</Params>
    ///<returns type="string">
    ///  Returns the new ASCII string
    ///</returns>
    ///<remarks>
    /// The conversion is done by looking at each char in the string and converting
    /// it into a range between 33 and 126.  If the char is already in that range it is
    /// passed through without any modification.  If the char is less than 33 then a char(32) is
    /// prefixed and the byte is added with 33 added onto it to bring it into the range.
    /// If the char is between 127 and 161, then char(32) is prefixed and the byte is
    /// added with 61 subtracted from it.
    /// If the char is greater than 161 then char(127) is prefixed and the byte is
    /// added with 129 subtracted from it.
    ///</remarks>
    string ConvertBytesToASCIIString(byte[] bp)
    {
      return Convert.ToBase64String( bp );

    } //*** ConvertBytesToASCIIString

    ///<summary>
    /// Converts a ASCII compatible (created using ConvertBytesToASCIIString
    /// string into a byte array
    ///</summary>
    ///<Params>
    ///  <param type="string" name="source" Required="">
    ///  string to convert
    ///  </param>
    ///</Params>
    ///<returns type="byte[]">
    ///  Returns the raw byte array
    ///</returns>
    byte[] ConvertASCIIStringToBytes(string source)
    {
      return Convert.FromBase64String(source);

    } //*** ConvertASCIIStringToBytes

  }
}