using System;
using System.Security.Principal;
using Vha.Vss.Utility;

namespace Vha.Vss.Security
{
  /// <summary>
  /// Logon types as defined in winbase.h.  These are exposed as enums for convenience and map
  /// directly to the #defines that begin with LOGON32_LOGON_xxx.
  /// </summary>
	public enum Win32LogonType 
  {
    /// <summary>Interactive</summary>
    Interactive=2,
    /// <summary>Network</summary> 
    Network,
    /// <summary>Batch</summary> 
    Batch,
    /// <summary>Service</summary> 
    Service,
    /// <summary>Unlock</summary> 
    Unlock=7,
    /// <summary>NetworkClearText</summary> 
    NetworkClearText,
    /// <summary>NewCredentials</summary> 
    NewCredentials 
  }
	
	/// <summary>
	/// Provider types as defined in winbase.h.  These are exposed as enums for convenience and map
	/// directly to the #defines that begin with LOGON32_PROVIDER_xxx.
	/// </summary>
	public enum Win32LogonProvider 
  {
    /// <summary>Default=0</summary>
    Default=0,
    /// <summary>WinNT35</summary> 
    WinNT35,
    /// <summary>WinNT40</summary> 
    WinNT40,
    /// <summary>WinNT50</summary> 
    WinNT50 
  }

  /// <summary>
  /// This is a wrapper class around some of the Win32 APIs in advapi.dll.  It provides
  /// the functions enabling an application to logon and impersonate a specified user
  /// and revert to previously logged on user.
  /// </summary>
  public class ImpersonationContext 
  {
    private String      m_UserName;
    private String      m_Password;
    private String      m_Domain;
    private IntPtr      m_NewUserToken;
    private IntPtr      m_PrevUserToken;
    private Boolean     m_IsPasswordEncrypted;

    private WindowsIdentity m_PrevUserIdentity;
		private VhaLog myLog;
    // We'll just expose these.  Not much point in a property for an enum (imho)
    /// <summary>
    /// Win32LogonType that is to be used for the logon performed prior to impersonation.  See MSDN for more specific details.
    /// </summary>
    public  Win32LogonType     LogonType;
    /// <summary>
    /// Win32LogonProvider that is to be used for the logon performed prior to impersonation.  See MSDN for more specific details.
    /// </summary>
    public  Win32LogonProvider LogonProvider;

    private ImpersonationContext (){} // hide it

    /// <summary>
    /// Constructor that will create and instance of the object initialized with
    /// a username and password.  
    /// </summary>
    /// <Params>
    /// <param name="userName" type="String">User name to be used to logon.  If domain is specifed the
    /// name will be parsed appropriately.  If not included the default domain for the executing application is used.
    /// </param>
    /// <param name="password" type="String">Password to be used for logon.  If the password is
    /// wrapped with '[' and ']' it will be considered encrypted and the value between the
    /// brackets will be decrypted with the utility DataEncoder class with a seed value of "eSupport".
    /// </param>
    /// </Params>
    /// <remarks>This is identical to ImpersonationContext(user, pwd, false);
    /// LogonType defaults to Win32LogonType.Interactive.  LogonProvider defaults to Win32LogonProvider.Default.
    /// </remarks>
    public ImpersonationContext (string userName, string password)
      : this(userName, password, Win32LogonType.Interactive, Win32LogonProvider.Default)
    {}

    /// <summary>
    /// Constructor that initializes every required property explicity.
    /// </summary>
    /// <Params>
    /// <param name="userName" type="String">User name to be used to logon.  If domain is specifed the
    /// name will be parsed appropriately.  If not included the default domain for the executing application is used.
    /// </param>
    /// <param name="password" type="String">Password to be used for logon.  If the password is
    /// wrapped with '[' and ']' it will be considered encrypted and the value between the
    /// brackets will be decrypted with the utility DataEncoder class with a seed value of "eSupport".
    /// </param>
    /// <param name="logonType">Logon type to be used.  See MSDN for more details.</param>
    /// <param name="logonProvider">Logon provider to be used.  See MSDN for more details.</param>
    /// </Params>
    /// <remarks>
    /// <para>ImpersonationContext(user, pwd) is equivilent to ImpersonationContext(user, pwd, Win32LogonType.Interactive, Win32LogonProvider.Default, false).</para>
    /// </remarks>
    public ImpersonationContext (string userName, string password, Win32LogonType logonType, Win32LogonProvider logonProvider)
    {
      UserName        = userName;   // use the property so Domain gets split out
      Password        = password;   // use the property so the m_IsPasswordEncrypted flag gets set
      LogonType       = logonType;
      LogonProvider   = logonProvider;
      m_NewUserToken  = IntPtr.Zero;
      m_PrevUserToken = IntPtr.Zero;
      m_PrevUserIdentity = null;
    }

    /// <summary>
    /// Begins running the current thread under the creditials identified by the class properties.
    /// The password i
    /// </summary>
    /// <Params>None.</Params>
    /// <returns type="void">None.</returns>
    /// <remarks>
    /// <para>Impersonation is implemented as first decrypting the password if requested, 
    /// logging on the provided username using the Win32 LogonUser(..) API and finally 
    /// performing the impersonation by a call to ImpersonateLoggedOnUser(..).
    /// </para>
    /// <para>
    /// If the call to the Win32 API fails an eSupportSecurityException will be thrown.
    /// </para>
    /// </remarks>
    public void BeginImpersonate()
    {
      String tmpPassword = Password;
      m_PrevUserIdentity = WindowsIdentity.GetCurrent();
      m_PrevUserToken = WindowsIdentity.GetCurrent().Token;

      if (m_IsPasswordEncrypted)
      {
        DataEncoder x = new DataEncoder();
        tmpPassword = x.DecodeString(m_Password.Substring(1, m_Password.Length-2));
      }
      if (Win32Wrapper.LogonUser(UserName, Domain, tmpPassword, (int)LogonType, (int)LogonProvider, out m_NewUserToken))
      {
        if (!Win32Wrapper.ImpersonateLoggedOnUser(m_NewUserToken))
        {
          throw new VhaSecurityException("Impersonation failed for User: " + Domain + "\\" + UserName);
        }
      }
      else
      {
        throw new VhaSecurityException("Logon failed for User: " + Domain + "\\" + UserName);
      }
    }

    /// <summary>
    /// Reverts the thread to run under the user context that was in place prior 
    /// to the call to BeginImpersonate.  The handles created by the logon process
    /// are released also.
    /// </summary>
    /// <Params>None.</Params>
    /// <returns type="void">None.</returns>
    /// <remarks>
    /// If the call to the Win32 API fails an eSupportSecurityException will be thrown.
    /// </remarks>
    public void EndImpersonate()
    {
      WindowsIdentity currIdent = null;
      String msg = "";
      if (m_PrevUserIdentity == null) return; // BeginImpersonate must have failed or EndImpersonate has already been called.  Be kind.
      if (!Win32Wrapper.ImpersonateLoggedOnUser(m_PrevUserIdentity.Token))
      {
        currIdent = WindowsIdentity.GetCurrent();
        msg = "ImpersonateLoggedOnUser(m_PrevUserIdentity.Token) failed attempting to revert to Previous User.\n";
        msg += String.Format("Current User: {0}\nIsAuthenticated: {1}\nIsSystem: {2}\nIsAnonymous: {3}\n",
          currIdent.Name, currIdent.IsAuthenticated, currIdent.IsSystem, currIdent.IsAnonymous);
        //myLog.Write(0, VhaLogType.Error, msg);
        throw new VhaSecurityException(msg);
      }
      if (m_NewUserToken != IntPtr.Zero) Win32Wrapper.CloseHandle(m_NewUserToken);
      m_NewUserToken = IntPtr.Zero;
      m_PrevUserIdentity = null;
    }

    #region Properties Get/Sets

    /// <summary>
    /// Username to be used for logging on.  If of the form DOMAIN\USER the domain
    /// name will be parsed out.
    /// </summary>
    public String UserName
    {
      get { return m_UserName; }
      set 
      {
        int i = value.IndexOfAny( new Char[2] {'/', '\\'});
        if (i>0)
        {
          m_Domain   = value.Substring(0,i);
          m_UserName = value.Substring(i+1);
        }
        else
        {
          m_UserName = value;
        }
      }
    }

    /// <summary>
    /// Password to be used for the impersonation logon.  If the m_IsPasswordEncrypted 
    /// property is True this value will be decrypted prior to use.
    /// </summary>
    public String Password
    {
      get { return m_Password; }
      set 
      { 
        m_Password = value; 
        m_IsPasswordEncrypted = m_Password.StartsWith("[") && m_Password.EndsWith("]");
      }
    }

    /// <summary>
    /// Domain of the user account to be used for logon.  Setting UserName property
    /// with a fully qualified DOMAIN/User name will set this property appropriately.
    /// </summary>
    public String Domain
    {
      get { return m_Domain; }
      set { m_Domain = value; }
    }
    #endregion

  }
}
