using System;
using System.Data;
using System.Xml;
using Vha.Vss.Utility;

namespace Vha.Vss.Utility
{
	/// <summary>
	/// XmlDataSource is a common class that encapsulates the properties that would be 
	/// used in the eSupport controls and XmlTransformation classes to identify and 
	/// obtain XML and XSLT data from files and databases.
	/// </summary>
	public class XmlDataSource
	{
    private String  m_Filename;
    private String  m_AppConfigFilename;
    private String  m_XPathQuery;
    private String  m_DBConnectionName;
    private String  m_StoredProcedure;
    private String  m_SqlQuery;

    private XmlDocument m_XmlDocumentObject;

    /// <summary>
    /// Default constructor. Does nothing and leaves the internal properties null;
    /// </summary>
		public XmlDataSource() {}

    /// <summary>
    /// This function verifies the established properties then loads the XmlDocument
    /// object from the specified data source (filename or database connection).  
    /// If a XmlDocumentObject has already been supplied no action is taken.
    /// </summary>
    public void InitializeDocumentObject()
    {
      if (XmlDocumentObject != null) return;  // don't load it more than once.

      // Make sure we have what we need for a data source (and nothing that would confuse us)
      if (DBConnectionName  == null && 
          Filename          == null &&
          XmlDocumentObject == null)
      {
        throw new VhaException(
          "You have not specified a Xml source.  You must supply a DBConnectionName, XmlFilename or " +
          "provide a preloaded XmlDocumentObject.  Please specify exactly one to have this control " +
          "class retrieve your data.  To bypass this feature override the Control_Init() " +
          "event and do not call the base class implementation.\n");
      }
      if (((SqlQuery != null && StoredProcedure != null) ||
           (SqlQuery == null && StoredProcedure == null))&&
           DBConnectionName != null)
      {
        throw new VhaException(
          String.Format("When you specify a DBConnectionName you must supply either a " +
          "SqlQuery or a StoredProcedure and you cannot supply both. " +
          "SqlQuery: {0}\nXmlDataSourceParms.StoredProcedure: {1}", 
          SqlQuery==null?"[null]" : SqlQuery, 
          StoredProcedure==null?"[null]" : StoredProcedure));
      }

      try
      {
        if (Filename != null)
        {
          //  A filename was specified so load it up
          //  Use the member variable m_XmlDocumentObject not the property so the filename doesn't get whacked
          m_XmlDocumentObject = new XmlDocument();  
          m_XmlDocumentObject.Load(Filename);
        }
      }
      catch(Exception ex)
      {
        throw new VhaException("Caught an Exception in XmlDataSource::InitializeDocumentObject():\n" + ToString(), ex);
      }
    }
    /// <summary>
    /// Returns a string representation of the object and all of its properties.
    /// </summary>
    /// <returns>String containing the contents of the objects properties.</returns>
    public override String ToString()
    {
      String Output = base.ToString() + "\n";
      try
      {
        Output += String.Format(
          "XmlDataSource properties...\n" +
          "AppConfigFilename: {0}\n"     +
          "DBConnectionName: {1}\n"      +
          "Filename: {2}\n"              +
          "SqlQuery: {3}\n"              +
          "StoredProcedure: {4}\n"       +
          "XmlDocumentObject: {5}\n"     +
          "XPathQuery: {6}\n",
          AppConfigFilename==null?"[null]" : AppConfigFilename,
          DBConnectionName==null ?"[null]" : DBConnectionName,
          Filename==null         ?"[null]" : Filename,
          SqlQuery==null         ?"[null]" : SqlQuery,
          StoredProcedure==null  ?"[null]" : StoredProcedure,
          XmlDocumentObject==null?"[null]" : "[OK]",
          XPathQuery==null       ?"[null]" : XPathQuery);
      }
      catch(Exception ex)
      {
        Output += "XmlDataSource data not available due to exception: " + ex.Message;
      }
      return Output;
    }
    #region Property Get/Set routines 
    /// <summary>
    /// String property designating a file to be used as the source for the XML.  The
    /// path can be relative or a fully qualified UNC, URL or drive pathname.  If the 
    /// Filename begins with "\\" or contains a colon (":") it will be considered fully-
    /// qualified and will be used verbatim.  Otherwise the path will be converted to 
    /// a fully qualified path under the application vroot path.
    /// </summary>
    ///<remarks>
    ///Setting the Filename property will reset the XmlDocument and DBConnectionName
    ///properties since these are mutually exclusive.
    ///</remarks>
    public String Filename
    {
      get { return m_Filename; }
      set 
      { 
        m_Filename          = value; 
        m_XmlDocumentObject = null;
        m_DBConnectionName  = null;
      }
    }

    /// <summary>
    /// Application configuration file to be used instead of the default web.config file.
    /// Typically this is not necessary for most applications and is only relevant to
    /// admin tools.  If not supplied the default "app.config" file will be used as
    /// determined by the URT.
    /// </summary>
    public String AppConfigFilename
    {
      get { return m_AppConfigFilename; }
      set { m_AppConfigFilename = value; }
    }

    /// <summary>
    /// Query to be run against the source of the XML.  It doesn't matter whether the
    /// XML source is database query or a text file.  In both cases the resulting 
    /// XmlNodeList will be obtained by running this query.  If not supplied the default
    /// query "/*/*" will be used. 
    /// </summary>
    public String XPathQuery
    {
      get { return m_XPathQuery; }
      set { m_XPathQuery = value; }
    }
    /// <summary>
    /// Name of a standard eSupport &lt;DBConnection&gt; that is defined in the application's
    /// web.config file.  This connection will be used to run the specified StoredProcedure
    /// or SqlQuery to obtain the Xml data.  
    /// </summary>
    ///<remarks>
    ///Setting the DBConnectionName property will reset the XmlDocument and XmlFilename
    ///properties since these are mutually exclusive.
    ///</remarks>
    public String DBConnectionName
    {
      get { return m_DBConnectionName; }
      set 
      { 
        m_DBConnectionName  = value; 
        m_Filename       = null;
        m_XmlDocumentObject = null;
      }
    }
    /// <summary>
    /// When a DBConnectionName is specified, this stored procedure will be invoked
    /// using the connection obtained from the web.config file.  The results of this
    /// query will still be filtered by the XPathQuery.  When specifying a StoredProcedure
    /// do not specify a SqlQuery.
    /// </summary>
    public String StoredProcedure
    {
      get { return m_StoredProcedure; }
      set { m_StoredProcedure = value; }
    }
    /// <summary>
    /// When a DBConnectionName is specified, this SQL query will be invoked
    /// using the connection obtained from the web.config file.  The results of this
    /// query will still be filtered by the XPathQuery.  When specifying a SqlQuery
    /// do not specify a StoredProcedure.
    /// </summary>
    public String SqlQuery
    {
      get { return m_SqlQuery; }
      set { m_SqlQuery = value; }
    }

    
    /// <summary>
    /// For use with code-behind to set a custom xml document that 
    /// is created dynamically at runtime.
    /// </summary>
    ///<remarks>
    ///Setting the XmlDocumentObject property will reset the XmlFilename and DBConnectionName
    ///properties since these are mutually exclusive.
    ///</remarks>
    public XmlDocument XmlDocumentObject
    {
      get { return m_XmlDocumentObject; }
      set 
      { 
        m_XmlDocumentObject = value; 
        m_Filename = null;
        m_DBConnectionName = null;
      }
    }
    
    #endregion


	}
}
