using System;
using System.Xml;
using System.Xml.Xsl;
using Vha.Vss.Utility;

namespace Vha.Vss.Utility
{
	/// <summary>
	/// Summary description for XmlDataSource.
	/// </summary>
	public class XslDataSource
	{
    private String        m_Filename;
    private XslTransform  m_XslTransformObject;

    /// <summary>
    /// Default constructor. Initializes the internal properties.
    /// </summary>
		public XslDataSource() {}

    /// <summary>
    /// This function verifies the established properties then loads the XslTransform
    /// object from the specified data source (filename).  If a XslTransformObject has
    /// already been supplied no action is taken.
    /// </summary>
    public void InitializeDocumentObject()
    {
      // Make sure we have what we need for a data source (and nothing that would confuse us)
      if (Filename == null && XslTransformObject == null)
      {
        throw new VhaException(
          "You have not specified a Xsl source.  You must supply a XmlFilename or " +
          "provide a preloaded XslTransformObject.  Please specify exactly one to have this control " +
          "class retrieve your data.  To bypass this feature override the Control_Init() " +
          "event and do not call the base class implementation.\n");
      }
      try
      {
        if (Filename != null)
        {
          //  A filename was specified so load it up
          m_XslTransformObject = new XslTransform();
          m_XslTransformObject.Load(Filename);
        }
      }
      catch(Exception ex)
      {
        throw new VhaException("Caught an Exception in XslDataSource::InitializeDocumentObject():\n" + ToString(), ex);
      }
    }

    /// <summary>
    /// Returns a string representation of the object and all of its properties.
    /// </summary>
    /// <returns>String containing the contents of the objects properties.</returns>
    public override String ToString()
    {
      String Output = base.ToString() + "\n";
      try
      {
        Output += String.Format(
          "XslDatSource properties...\n" +
          "Filename: {0}\n"              +
          "XslTransformObject: {1}\n",
          Filename==null         ?"[null]" : Filename,
          XslTransformObject==null?"[null]" : "[OK]");
      }
      catch(Exception ex)
      {
        Output += "XslDataSource data not available due to exception: " + ex.Message;
      }
      return Output;
    }

    #region Property Get/Set routines 

    /// <summary>
    /// String property designating a file to be used as the source for the XSLT.  The
    /// path can be relative or a fully qualified UNC, URL or drive pathname.  If the 
    /// Filename begins with "\\" or contains a colon (":") it will be considered fully-
    /// qualified and will be used verbatim.  Otherwise the path will be converted to 
    /// a fully qualified path under the application vroot path.
    /// </summary>
    ///<remarks>
    ///Setting the XslFilename property will reset the XslDocument 
    ///property since these are mutually exclusive.
    ///</remarks>
    public String Filename
    {
      get { return m_Filename; }
      set 
      { 
        m_Filename           = value; 
        m_XslTransformObject = null;
      }
    }

    /// <summary>
    /// For use with code-behind to set a custome xslt document at runtime.
    /// </summary>
    ///<remarks>
    ///Setting the XslDocumentObject property will reset the XslFilename 
    ///property since these are mutually exclusive.
    ///</remarks>
    public XslTransform XslTransformObject
    {
      get { return m_XslTransformObject; }
      set 
      { 
        m_XslTransformObject = value; 
        m_Filename = null;
      }
    }
   
    #endregion

	}
}
