using System;
using System.Collections;
using System.Data;
using System.Windows.Forms;

namespace Vha.Vtk.Kernel.Common
{
	/// <summary>
	/// Summary description for ReturnStatus.
	/// </summary>
	
	/// <summary>
	/// Enumeration that is used to indicate the success level of a method call.
	/// </summary>
	public enum ReturnStatusIndicator 
	{
		/// <summary>
		/// Operation completed and was successful
		/// </summary>
		Pass,
		/// <summary>
		/// Operation ran but failed
		/// </summary>
		Fail,
		/// <summary>
		/// General erorr with no additional information (This enum is currently not used.)
		/// </summary>
		Error,
		/// <summary>
		/// Specific error code (This enum is currently not used.)
		/// </summary>
		Code}

	/// <summary>
	/// Class that defines the return value for validation methods. Applications calling
	/// validation methods then determine the next action.
	/// </summary>
	public class ReturnStatus 
	{
		#region Section: Processing Fields

		/// <summary>
		/// Overall status of method upon return.
		/// </summary>
		private ReturnStatusIndicator status;

		/// <summary>
		/// Error code number (Not currently used)
		/// </summary>
		private int errorCode;

		/// <summary>
		/// Error message returned.
		/// </summary>
		private string errorText;

		/// <summary>
		/// Class name of instance being validated.
		/// </summary>
		private string className;

		/// <summary>
		/// Property name being validated.
		/// </summary>
		private string propertyName;

		#endregion

		#region Section: Property Definitions

		/// <summary>
		/// Overall status of method upon return.
		/// </summary>
		/// <value type="ReturnStatusIndicator"></value>
		public ReturnStatusIndicator Status
		{
			get {return status;}
			set {status = value;}
		}

		/// <summary>
		/// Error code number (Not currently used)
		/// </summary>
		/// <value type="System.Int"></value>
		public int ErrorCode
		{
			get {return errorCode;}
			set {errorCode = value;}
		}

		/// <summary>
		/// Error message returned.
		/// </summary>
		/// <value type="System.String"></value>
		public string ErrorText
		{
			get {return errorText;}
			set {errorText = value;}
		}
		
		/// <summary>
		/// Class name of instance being validated.
		/// </summary>
		/// <value type="System.String"></value>
		public string ClassName
		{
			get {return className;}
			set {className = value;}
		}
		
		/// <summary>
		/// Property name being validated.
		/// </summary>
		/// <value type="System.String"></value>
		public string PropertyName
		{
			get {return propertyName;}
			set {propertyName = value;}
		}
		
		#endregion

		#region Section: Constructors

		/// <summary>
		/// Main constructor.
		/// </summary>
		public ReturnStatus() : base()
		{
			Clear();
		}
		
		#endregion

		#region Section: Utilities

		/// <summary>
		/// Initializes/Resets properties to default values.
		/// </summary>
		/// <remarks>
		/// The default values are the following:
		/// <list type="bullet">
		/// <item><description>
		///    status       = ReturnStatusIndicator.Pass
		/// </description></item>
		/// <item><description>
		///    errorCode    = 0
		/// </description></item>
		/// <item><description>
		///    errorText    = No Errors
		/// </description></item>
		/// <item><description>
		///    className    = ""
		/// </description></item>
		/// <item><description>
		///    propertyName = ""
		/// </description></item>
		/// </list>
		/// </remarks>
		public void Clear()
		{
			status       = ReturnStatusIndicator.Pass;
			errorCode    = 0;
			errorText    = "<No Errors>";
			className    = "";
			propertyName = "";
		}

		/// <summary>
		/// Displays the error message in a standard Windows message box.
		/// </summary>
		public void DisplayErrorInfo()
		{
			MessageBox.Show(ErrorText);
		}


		/// <summary>
		/// Returns status property as string.
		/// </summary>
		/// <returns type="System.String"></returns>
		public override string ToString()
		{
			if (status == ReturnStatusIndicator.Pass)       {return "Pass";}
			if (status == ReturnStatusIndicator.Error)      {return "Error";}
			if (status == ReturnStatusIndicator.Code)       {return "Code";}
			if (status == ReturnStatusIndicator.Fail)       {return "Fail";}
			return "Unknown";
		}

		#endregion
	}
}
