using System;
using System.Collections;
using System.Data;
using System.Windows.Forms;

namespace Vha.Vtk.Kernel.Common
{
	/// <summary>
	/// Summary description for SelectCriteria.
	/// </summary>
	
	/// <summary>
	/// This class is used to help control long list of look up entries being passed
	/// from the server to the client application.
	/// </summary>
	[System.Serializable()]
	public class SelectCriteria
	{
		#region Section: Processing Fields
		/// <summary>
		/// Specifies the record to start at.
		/// </summary>
		private int startAt;
		/// <summary>
		/// Specifies the record to end at.
		/// </summary>
		private int endAt;
		/// <summary>
		/// Private field used to store tableName value.
		/// This field is accessed using the TableName property.
		/// </summary>
		private string tableName;

		/// <summary>
		/// Private field used to store startsWith value.
		/// This field is accessed using the StartsWith property.
		/// </summary>
		private string startsWith;

		/// <summary>
		/// Private field used to store startNextFetchAt value.
		/// This field is accessed using the StartNextFetchAt property.
		/// </summary>
		private string startNextFetchAt;

		/// <summary>
		/// Private field used to store crossReference value.
		/// This field is accessed using the CrossReference property.
		/// </summary>
		private string crossReference;

		/// <summary>
		/// Private field used to store count value.
		/// This field is accessed using the Count property.
		/// </summary>
		private int    count;

		/// <summary>
		/// Private field used to store maxRecords value.
		/// This field is accessed using the MaxRecords property.
		/// </summary>
		private int    maxRecords;

		/// <summary>
		/// Private field used to store atEnd value.
		/// This field is accessed using the AtEnd property.
		/// </summary>
		private bool   atEnd;

		#endregion

		#region Section: Property Definitions
		/// <summary>
		/// Specifies the record number to start at.
		/// </summary>
		public int StartAt
		{
			get{return startAt; }
			set{startAt = value; }
		}
		/// <summary>
		/// Specifies the record number to end at.
		/// </summary>
		public int EndAt
		{
			get{return endAt; }
			set{endAt = value; }
		}
		/// <summary>
		/// Name of the table assigned to the list of entries retrived.
		/// This property typically is assigned by the server.
		/// </summary>
		public string TableName
		{
			get {return tableName;}
			set {tableName = value;}
		}
		
		/// <summary>
		/// Text used as the starting point for the look up on the initial fetch.
		/// This property is set by the client application.
		/// </summary>
		public string StartsWith
		{
			get 
			{
				return startsWith;
			}
			set 
			{
				startsWith = value;
			}
		}
		
		/// <summary>
		/// Token value assigned by the server look up code that indicates where the
		/// next fetch should start when the client asks for more entries.
		/// 
		/// This property is totally under the control of the server code.
		/// </summary>
		public string StartNextFetchAt
		{
			get
			{
				return startNextFetchAt;
			}
			set
			{
				startNextFetchAt = value;
			}
		}
		// TODO: Code Enhancement Possibility: (5) Can 'CrossReference' be eliminated? It was used for legacy chunkers.
		/// <summary>
		/// This property can indicate which cross reference value the server look up code
		/// is using.
		/// 
		/// This property is informational only at this time but could be used to help
		/// the client application tell the server code exactly which cross reference to
		/// use. 
		/// </summary>
		public string CrossReference
		{
			get 
			{
				return crossReference;
			}
			set 
			{
				crossReference = value;
			}
		}
		
		/// <summary>
		/// Count of the numner of recounts retrieved by the most recent fetch.
		/// 
		/// This property is assigned by the server.
		/// </summary>
		public int Count
		{
			get 
			{
				return count;
			}
			set 
			{
				count = value;
			}
		}

		/// <summary>
		/// Maximun number of records that should be fetched.
		/// 
		/// This property is assigned but the client. [default = 9999]
		/// </summary>
		public int MaxRecords
		{
			get
			{
				return maxRecords;
			}
			set 
			{
				maxRecords = value;
			}
		}

		/// <summary>
		/// This boolean property indicates whether the most recent fetch reached the end of
		/// list of entries.
		/// </summary>
		public bool AtEnd
		{
			get 
			{
				return atEnd;
			}
			set 
			{
				atEnd = value;
			}
		}


		#endregion

		#region Section: Constructors
		/// <summary>
		/// Main constructor for SelectCriteria class.
		/// </summary>
		public SelectCriteria()
		{
			Clear();
		}


		#endregion

		#region Section: Utilities
		/// <summary>
		/// Sets properties for a SelectCriteria instance to default values.
		/// </summary>
		/// <remarks>
		/// The default values are the following:
		/// <list type="bullet">
		/// <item><description>
		///    TableName        = Unknown
		/// </description></item>
		/// <item><description>
		///    StartsWith       = ""
		/// </description></item>
		/// <item><description>
		///    StartNextFetchAt = ""
		/// </description></item>
		/// <item><description>
		///    CrossReference   = ""
		/// </description></item>
		/// <item><description>
		///    Count            = 0
		/// </description></item>
		/// <item><description>
		///    MaxRecords       = 9999
		/// </description></item>
		/// <item><description>
		///    AtEnd            = false
		/// </description></item>
		/// </list>
		/// </remarks>
		public void Clear()
		{
			tableName        = "Unknown";
			startsWith       = "";
			startNextFetchAt = "";
			crossReference   = "";
			count            = 0;
			maxRecords       = 9999;
			startAt					 = 0;
			endAt						 = 10;
			atEnd            = false;
		}

		/// <summary>
		/// This method returns a string array that is typically needed for VA RPC Broker calls
		/// related to look up RPCs.
		/// </summary>
		/// <returns></returns>
		public string[] SelectArray()
		{
			return new string[] {startsWith,
														startNextFetchAt,
														crossReference,
														maxRecords.ToString(),
														startAt.ToString(),
														endAt.ToString()};
		}

		/// <summary>
		/// This method sets the state of the SelectCriteria based on the XobTableSummary entry.
		/// This entry is created and sent back as part of a look up fetch from the server.
		/// </summary>
		/// <param name="dr"></param>
		public void SetState(DataRow dr)
		{
			tableName        = dr["TableName"].ToString();
			startsWith       = dr["StartsWith"].ToString();
			startNextFetchAt = dr["StartNextFetchAt"].ToString();
			crossReference   = dr["CrossReference"].ToString();
			count            = Convert.ToInt32(dr["Count"].ToString());
			maxRecords       = Convert.ToInt32(dr["MaxRecords"].ToString());
			atEnd            = Convert.ToBoolean(dr["AtEnd"].ToString());
		}

		#endregion
	}
}
