using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using System.Web.UI.Design;
using System.Web.Security;
using Vha.Vtk.Kernel.Data.SystemTables;
using Vha.Vtk.TimeKeeping;
using System.Text.RegularExpressions;
using System.Data;

[ assembly:TagPrefix("Vha.Vtk.WebControlLibrary", "VhaVtk") ]
namespace Vha.Vtk.WebControlLibrary
{
	/// <summary>
	/// <para>This web control is for use by the VtkUser application. It should be used by ALL pages within the 
	/// VtkUser application where a volunteer is considered "logged in". This means all pages except for the 
	/// first (login.aspx) and last (thankyou.aspx).</para>
	/// <para>This control maintains information about the currently selected language for the volunteer. It also
	/// displays the station name and volunteer name. Finally, it provides a "log out" button.</para>
	/// </summary>
	[ToolboxData("<VhaVtk:VtkUserPageHeader runat=server></VhaVtk:VtkUserPageHeader>"),
	Designer("Vha.Vtk.WebControlLibrary.VtkUserPageHeaderDesigner, Vha.Vtk.WebControlLibrary")]
	public class VtkUserPageHeader : Control, INamingContainer
	{
		const string STATIONID = "StationId";

		const string INFOVOLFULLNAMEFORREPORTS="VhaPageHeaderVolFullNameForReports";
		const string INFOSTATIONDISPLAYNAME="VhaPageHeaderStationDisplayName";
		const string INFOSTATIONID="VhaPageHeaderStationDisplayId";
		const string INFOLANGUAGE="VhaPageHeaderLanguage";

		/// <summary>
		/// Use to set the language that the control should consider the current language. Currently 
		/// supporting "es" and "en-US."  It is populated only after 
		/// <see cref="VtkUserPageHeader.VerifyStation"/>() is called.
		/// </summary>
		[Bindable(true), Category("Misc"), DefaultValue(""), ReadOnly(false),
		Description("Set the language that the control should consider the current language. Currently supporting es and en-US.")]
		public string CurrentLanguage
		{
			get 
			{
				if (ViewState[INFOLANGUAGE]!=null) 
				{
					return (string) ViewState[INFOLANGUAGE];
				} 
				else 
				{
					return "en-US";
				}
			}
			set 
			{
				ViewState[INFOLANGUAGE]=value;
			}
		}

		/// <summary>
		/// Contains the volunteer's currently selected Station ID. It is populated only after 
		/// <see cref="VtkUserPageHeader.VerifyStation"/>() is called.
		/// </summary>
		[Bindable(false), Category("Data"), DefaultValue(-1), ReadOnly(true), 
		Description("The volunteer's currently selected station ID. It is populated only after VerifyStation() is called.")]
		public int CurrentStationId 
		{
			get 
			{
				if (ViewState[INFOSTATIONID]!=null) 
				{
					return (int) ViewState[INFOSTATIONID];
				} 
				else 
				{
					return -1;
				}
			}
			set 
			{
				ViewState[INFOSTATIONID]=value;
			}
		}

		/// <summary>
		/// Contains the volunteer's currently selected Station name (for display). It is populated only after 
		/// <see cref="VtkUserPageHeader.VerifyStation"/>() is called.
		/// </summary>
		[Bindable(false), Category("Data"), DefaultValue("not selected"), ReadOnly(true), 
		Description("The volunteer's currently selected Station name (for display). It is populated only after VerifyStation() is called.")]
		public string CurrentStationDisplayName
		{
			get 
			{
				if (ViewState[INFOSTATIONDISPLAYNAME]!=null) 
				{
					return (string) ViewState[INFOSTATIONDISPLAYNAME];
				} 
				else 
				{
					return "not selected";
				}
			}
			set 
			{
				ViewState[INFOSTATIONDISPLAYNAME]=value;
			}
		}

		/// <summary>
		/// Contains the volunteer's "full name for reports". It is populated only after 
		/// <see cref="VtkUserPageHeader.VerifyStation"/>() is called.
		/// </summary>
		[Bindable(false), Category("Data"), DefaultValue("not selected"), ReadOnly(true), 
		Description("The volunteer's FullNameForReports. It is populated only after VerifyStation() is called.")]
		public string CurrentVolFullNameForReports
		{
			get 
			{
				if (ViewState[INFOVOLFULLNAMEFORREPORTS]!=null) 
				{
					return (string) ViewState[INFOVOLFULLNAMEFORREPORTS];
				} 
				else 
				{
					return "not selected";
				}
			}
			set 
			{
				ViewState[INFOVOLFULLNAMEFORREPORTS]=value;
			}
		}

		/// <summary>
		/// Fired when a volunteer clicks the "change station" link.
		/// </summary>
		[Bindable(true), Category("Misc"), ReadOnly(false),
		Description("Fired when a volunteer clicks the Change Station link.")]
		public event EventHandler Click;

		/// <summary>
		/// In any page using the control, call VerifyStation() in the Page_Load method, in the 
		/// <c>if (!IsPostBack){}</c> section. This call sets up values for
		/// <see cref="VtkUserPageHeader.CurrentLanguage"/>, 
		/// <see cref="VtkUserPageHeader.CurrentStationId"/>, 
		/// <see cref="VtkUserPageHeader.CurrentStationDisplayName"/>, and
		/// <see cref="VtkUserPageHeader.CurrentVolFullNameForReports"/>,
		/// based on information
		/// passed to it in either Context.Items, or in URL parameters, from the preceding page. It also
		/// sets up what to display for station and volunteer name, in the control itself.
		/// </summary>
		public void VerifyStation() 
		{
			this.EnsureChildControls();
			this.CurrentStationId=-1;

			// Populate various properties from Context.Items.
			if (Context.Items[INFOSTATIONDISPLAYNAME]!=null)
			{
				//Populate properties from Context.Items
				CurrentLanguage=(string) Context.Items[INFOLANGUAGE];
				CurrentStationId=(int) Context.Items[INFOSTATIONID];
				CurrentStationDisplayName=(string)Context.Items[INFOSTATIONDISPLAYNAME];
				CurrentVolFullNameForReports=(string)Context.Items[INFOVOLFULLNAMEFORREPORTS];
				//Set the visible control values
				PopulateControls();
			} 
			else
			{
				// Nothing in Context.Items
				throw new Exception("Warning: No Station Number Found");
			}

			if (CurrentStationId<1) {
				// Station ID in Context.Items was invalid
				throw new Exception("Warning: No Station Number Found");
			}
		}

		//12/31/01 KC changed from public to private
		private void PopulateControls() 
		{
			((Label)Controls[1]).Text=this.CurrentStationDisplayName;
			((Label)Controls[3]).Text=this.CurrentVolFullNameForReports;
			if (this.CurrentLanguage=="es") 
			{
				((LinkButton)Controls[5]).Text="Fin de comunicacin/Salir";
			} 
			else 
			{
				((LinkButton)Controls[5]).Text="Log Out";
			}
		}

		//12/31/01 KC changed from public to private
		private void ContextItemsStuff() 
		{
			Context.Items[INFOLANGUAGE]=CurrentLanguage;
			Context.Items[INFOSTATIONDISPLAYNAME]=CurrentStationDisplayName;
			Context.Items[INFOSTATIONID]=CurrentStationId;
			Context.Items[INFOVOLFULLNAMEFORREPORTS]=this.CurrentVolFullNameForReports;
		}

		/// <summary>
		/// Performs the Server.Transfer() call to move to the specified page. Automatically appends required
		/// URL parameters to the URL. Also, automatically placess all needed information into
		/// Context.Items before the call, so that the VtkPageHeader control on the next page has all 
		/// the information it needs.
		/// </summary>
		/// <param name="page">Name of page to navigate to.</param>
		public void ServerTransfer(string page) 
		{
			//Stuff Context.Items
			ContextItemsStuff();
			//Do the server.tranfer() here
			Page.Server.Transfer(page+"?"+STATIONID+"="+this.CurrentStationId);
		}

		/// <summary>
		/// Returns the URL parameter in which the control always expects to contain the current station Id. 
		/// Use this method so that you know what URL parameter to place the station ID in, when building a URL
		/// to jump to (when the page being jumped to also uses the VtkUserPageHeader control).
		/// </summary>
		/// <returns></returns>
		public static string GetURLParameterString() 
		{
			return STATIONID;
		}

		/// <summary>
		/// This method is for internal use of the control only. It overrides the 
		/// <see cref="System.Web.UI.Control.CreateChildControls"/> method.
		/// </summary>
		protected override void CreateChildControls() 
		{
			Label StationLabel = new Label();
			StationLabel.Text="not selected";

			Label UserNameLabel = new Label();
			UserNameLabel.Text = "not selected";
			UserNameLabel.Attributes.Add("Style", "FONT-SIZE: 14pt;FONT-WEIGHT: 600;");
			LinkButton LogOutButton = new LinkButton();
			if (this.CurrentLanguage=="es") 
			{
				LogOutButton.Text="Fin de comunicacin";
			} 
			else 
			{
				LogOutButton.Text="Log Out";
			}

			LogOutButton.CommandName="Click";

			Controls.AddAt(0,new LiteralControl("<table bgcolor=\"#eeeeee\" width=\"100%\" border=0 cellspacing=2 cellpadding=0><tr><td align=\"left\" width=\"25%\">"));
			Controls.AddAt(1, StationLabel);
			Controls.AddAt(2,new LiteralControl("</td><td align=\"center\" width=\"50%\">"));
			Controls.AddAt(3, UserNameLabel);
			Controls.AddAt(4,new LiteralControl("</td><td align=\"right\" width=\"25%\">"));
			Controls.AddAt(5, LogOutButton);
			Controls.AddAt(6,new LiteralControl("</td></tr></table>"));

			StationLabel.EnableViewState=true;
			UserNameLabel.EnableViewState=true;
		}

		/// <summary>
 		/// Helps allow the "log out" link to generate a Click event for the control.
		/// </summary>
		/// <param name="source"></param>
		/// <param name="e"></param>
		/// <returns></returns>
		protected override bool OnBubbleEvent(object source, EventArgs e) 
		{
			bool handled=false;
			if (e is CommandEventArgs) 
			{
				CommandEventArgs ce = (CommandEventArgs) e;
				if (ce.CommandName == "Click") 
				{
					OnClick(ce);
					handled=true;
				}
			}
			return handled;
		} 

		/// <summary>
		/// Helps allow the "log out" link to generate a Click event for the control. Called from
		/// <see cref="VtkUserPageHeader.OnBubbleEvent"/>.
		/// </summary>
		/// <param name="e"></param>
		protected virtual void OnClick(EventArgs e) 
		{
			if (Click != null) 
			{
				// log out
				FormsAuthentication.SignOut();
				Page.Response.Redirect("login.aspx?"+STATIONID+"="+this.CurrentStationId);
			}
		} 
	}

	/// <summary>
	/// Used by the <see cref="Vha.Vtk.WebControlLibrary.VtkUserPageHeader"/> control to display the control at design-time (in the web form
	/// designer). It shows enough of what the control will look like to make designing web forms
	/// easier.
	/// </summary>
	public class VtkUserPageHeaderDesigner : System.Web.UI.Design.ControlDesigner 
	{
		/// <summary>
		/// This method is called by VS.NET to display the control at design-time (in the web form
		/// designer). It shows enough of what the control will look like to make designing web forms
		/// easier.
		/// </summary>
		/// <returns>HTML string to display a facsimile of the control in the web page designer.</returns>
		public override string GetDesignTimeHtml()
		{
			//TODO: make this aware of the language setting?
			return "<table width=\"100%\" bgcolor=\"#eeeeee\"><tr><td align=\"left\" width=\"25%\">Station DisplayName</td><td align=\"center\" width=\"50%\" Style=\"FONT-SIZE: 12pt;FONT-WEIGHT: bold;\">Volunteer Name</td><td align=\"right\" width=\"25%\"><a href=\"\">Log Out</a></td></tr></table>";
		}
	}

}
